#include <jagdefs.h>
#include <jagtypes.h>
#include <stdlib.h>

#include <interrupt.h>
#include <display.h>
#include <sound.h>

#include <joypad.h>

#include <sprite.h>
#include <screen.h>
#include <fb2d.h>

extern phrase removers_logo;
extern char muzaxx;

// this is a valid blitter width
#define LOGO_WIDTH 256 
#define LOGO_HEIGHT 64

// this is a valid blitter width
#define MAXWIDTH 320
#define MAXHEIGHT 320

/*star struct*/
typedef struct
{
  short int xpos;
  short int ypos;
  short int zpos;
  short int speed;
  //  int offset;
} STAR;

//#define NB_STARS 128

#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 240

#define CENTERX (SCREEN_WIDTH / 2)
#define CENTERY (SCREEN_HEIGHT / 2)

//static STAR stars[NB_STARS];

//extern STAR stars[NB_STARS];
//extern int precalc[63*16+1]; 

#include "starfield.h"

static void *starfield_rout_addr = NULL;

int main(int argc, char *argv[]) {
  int alpha;
  int w, h;

  TOMREGS->vmode = RGB16|CSYNC|BGEN|PWIDTH4|VIDEN;
  init_interrupts();
  init_display_driver();
  init_sound_driver(16000);

  init_module(&muzaxx,1);

  display *d = new_display(0);
  d->x = 16;
  d->y = 8;

  void *fb2d_addr = &_DSP_FREE_RAM;
  void *dsp_available_ram = init_fb2d_manager(fb2d_addr);
  affine_transform *m = malloc(sizeof(affine_transform));
  linear_transform *n = malloc(sizeof(linear_transform));

  screen *removers_scr = new_screen();
  set_simple_screen(DEPTH16,LOGO_WIDTH,LOGO_HEIGHT,removers_scr,&removers_logo);

  screen *fb_scr = new_screen();
  phrase *fb = alloc_simple_screen(DEPTH16,MAXWIDTH,MAXHEIGHT,fb_scr);
  clear_screen(fb_scr);

  sprite *fb_sprite = sprite_of_screen((SCREEN_WIDTH-LOGO_WIDTH)/2,-MAXHEIGHT/2,fb_scr);
  attach_sprite_to_display_at_layer(fb_sprite,d,0);

  show_display(d);

  vblQueue[0] = play_module;

  // 2048 is 2 * pi
  for(alpha = 2048/4; (alpha >= 0); alpha--) {
    fb2d_set_rotation((linear_transform *)m,alpha);
    n->a = LOGO_WIDTH;
    n->b = 0;
    n->c = 0;
    n->d = 0;
    fb2d_compose_linear_transform((linear_transform *)m,n);
    fb2d_compute_bounding_box((linear_transform *)m,LOGO_WIDTH,LOGO_HEIGHT,&w,&h);
    if(h > MAXHEIGHT) {
      h = MAXHEIGHT;
    }
    fb_scr->y = MAXHEIGHT - h;
    fb2d_set_matching_points(m,LOGO_WIDTH,LOGO_HEIGHT,n->a,h);
    vsync();
    fb2d_copy_transformed(removers_scr,fb_scr,m,w,h,MODE_S);
  }

  sprite *removers_sprite = new_sprite(LOGO_WIDTH,LOGO_HEIGHT,(SCREEN_WIDTH-LOGO_WIDTH)/2,MAXHEIGHT/2-LOGO_HEIGHT,DEPTH16,&removers_logo);
  removers_sprite->hx = LOGO_WIDTH/2;
  removers_sprite->hy = LOGO_HEIGHT/2;
  removers_sprite->x = SCREEN_WIDTH/2;
  removers_sprite->y = MAXHEIGHT/2-LOGO_HEIGHT/2;
  removers_sprite->use_hotspot = 1;
  removers_sprite->scaled = 1;
  removers_sprite->hscale = 1<<5;
  removers_sprite->vscale = 1<<5;
  
  attach_sprite_to_display_at_layer(removers_sprite,d,1);
  detach_sprite_from_display(fb_sprite);
  free(fb);
  free(fb_sprite);
  free(fb_scr);

  vsync();

  int i;
  for(i = 0; i < 64; i++) {
    TOMREGS->clut1[64-i] = RGBCOLOR(4*i,4*i,4*i);
  }
  TOMREGS->clut1[0] = 0;

  starfield_rout_addr = (void *)&_GPU_FREE_RAM;
  init_starfield_routine(starfield_rout_addr);

  fb_scr = new_screen();
  fb = alloc_simple_screen(DEPTH8,SCREEN_WIDTH,SCREEN_HEIGHT,fb_scr);
  clear_screen(fb_scr);
  phrase *fb2 = alloc_simple_screen(DEPTH8,SCREEN_WIDTH,SCREEN_HEIGHT,fb_scr);
  clear_screen(fb_scr);
  fb_sprite = sprite_of_screen(0,0,fb_scr);

  attach_sprite_to_display_at_layer(fb_sprite,d,0);

  int tempx, tempy, temp;
  char *phys = (char *)fb2;
  char *log = (char *)fb;
  char *screen = log;

  int phase = 1;

  int scale = 1<<5;

  starfield_center_x = CENTERX;
  starfield_center_y = CENTERY;
  starfield_speedz = 8;

  joypad_state *j = malloc(sizeof(joypad_state));

  for(;;) {
    vsync();

    read_joypad_state(j);

    temp = j->j1;

    if(temp & JOYPAD_DOWN) {
      starfield_center_y++;
    } else if(temp & JOYPAD_UP) {
      starfield_center_y--;
    }

    if(temp & JOYPAD_LEFT) {
      starfield_center_x--;
    } else if(temp & JOYPAD_RIGHT) {
      starfield_center_x++;
    }

    if(temp & JOYPAD_A) {
      starfield_speedz--;
    } else if(temp & JOYPAD_C) {
      starfield_speedz++;
    }

    if(starfield_speedz < 0) { 
      starfield_speedz = 0;
    } else if(starfield_speedz > 16) {
      starfield_speedz = 16;
    }

    if(starfield_center_x < 0) {
      starfield_center_x = 0;
    } else if(starfield_center_x >= SCREEN_WIDTH) {
      starfield_center_x = SCREEN_WIDTH-1;
    }

    if(starfield_center_y < 0) {
      starfield_center_y = 0;
    } else if(starfield_center_y >= SCREEN_HEIGHT) {
      starfield_center_y = SCREEN_HEIGHT-1;
    }

    wait_display_refresh();

    screen = phys;
    phys = log;
    log = screen;
    fb_scr->data = log;
    fb_sprite->data = phys;

    clear_screen(fb_scr);

    starfield_address = screen;

    go_starfield(starfield_rout_addr);
    wait_starfield();
  }

  return 0;
}

